@@ -28,6 +28,7 @@ def matrix(
     link_seg: bool = True,
     feature_style: str = "normal",
     feature_spacing: float = 1,
+    cmap: Optional[str] = None,
     colorbar: bool = True,
     colorbar_title: Optional[str] = None,
     figsize: Union[None, tuple] = None,
@@ -61,6 +62,8 @@ def matrix(
         Font style of the feature labels.
     feature_spacing
         When figsize is None, controls the the height of each rows.
+    cmap
+        colormap to use, by default is plt.rcParams["image.cmap"].
     colorbar
         Show the colorbar.
     colorbar_title
@@ -83,6 +86,8 @@ def matrix(
     """
     adata = adata[:, features].copy()
 
+    cmap = plt.rcParams["image.cmap"] if cmap is None else cmap
+
     if (layer == "fitted") & ("fitted" not in adata.layers):
         print("Features not fitted, using X expression matrix instead")
         layer = None
@@ -143,8 +148,7 @@ def matrix(
         sharey=True,
         figsize=(
             len(order) * 0.85 + 0.85 + annot_var * 1,
-            (len(features) + 0.8 * annot_top + 1 + 0.8 * annot_var)
-            / (5 - feature_spacing),
+            (len(features)) / (5 - feature_spacing) + 0.4 * annot_top,
         )
         if figsize is None
         else figsize,
@@ -171,25 +175,26 @@ def matrix(
             logg.warn(f"removed {len(empty)} empty interval")
             adata_sub.obs.split = adata_sub.obs.split.cat.remove_unused_categories()
 
-        ss = int(adata.obs.seg.cat.categories[s])
-        sel = (
-            adata.obs.milestones.cat.categories
-            == vs2mils[graph["pp_seg"].loc[ss]["from"]]
-        )
-        start = np.array(adata.uns["milestones_colors"])[sel][0]
-        sel = (
-            adata.obs.milestones.cat.categories
-            == vs2mils[graph["pp_seg"].loc[ss]["to"]]
-        )
-        end = np.array(adata.uns["milestones_colors"])[sel][0]
+        if annot_top:
+            ss = int(adata.obs.seg.cat.categories[s])
+            sel = (
+                adata.obs.milestones.cat.categories
+                == vs2mils[graph["pp_seg"].loc[ss]["from"]]
+            )
+            start = np.array(adata.uns["milestones_colors"])[sel][0]
+            sel = (
+                adata.obs.milestones.cat.categories
+                == vs2mils[graph["pp_seg"].loc[ss]["to"]]
+            )
+            end = np.array(adata.uns["milestones_colors"])[sel][0]
 
-        my_cm = LinearSegmentedColormap.from_list("aspect", [start, end])
+            my_cm = LinearSegmentedColormap.from_list("aspect", [start, end])
 
         if "use_raw" not in kwargs:
             kwargs["use_raw"] = False
 
         M = sc.pl.MatrixPlot(
-            adata_sub, features, "split", vmin=0, vmax=maxval, **kwargs
+            adata_sub, features, "split", vmin=0, vmax=maxval, cmap=cmap, **kwargs
         )
         M.swap_axes()
         M._mainplot(axs[i])
@@ -308,8 +313,6 @@ def matrix(
             borderpad=2,
         )
 
-        cmap = plt.rcParams["image.cmap"] if "cmap" not in kwargs else kwargs["cmap"]
-
         mappable = cm.ScalarMappable(cmap=cm.get_cmap(cmap))
         cbar = fig.colorbar(mappable, cax=position, orientation="horizontal", aspect=50)
         cbar.set_ticks([0, 1])
@@ -320,12 +323,12 @@ def matrix(
             ]
         )
         title = "expression"
-        if norm == "minmax":
+        if colorbar_title is not None:
+            title = colorbar_title
+        elif norm == "minmax":
             title = "minmax\nnormalized\nexpression"
         elif norm == "max":
             title = "max\nnormalized\nexpression"
-        elif colorbar_title is not None:
-            title = colorbar_title
         cbar.ax.set_title(title, loc="center")
 
     savefig_or_show("matrix", show=show, save=save)
