@@ -1,4 +1,4 @@
-from typing import Optional, Union
+from typing import Optional, Union, Iterable
 from typing_extensions import Literal
 from anndata import AnnData
 import numpy as np
@@ -138,15 +138,19 @@ def cleanup(
 
 def subset_tree(
     adata: AnnData,
-    root_milestone,
-    milestones,
-    mode: Literal["extract", "substract"] = "extract",
+    root_milestone: Optional[str] = None,
+    milestones: Optional[Iterable] = None,
+    mode: Literal["extract", "substract", "pseudotime"] = "extract",
+    t_min: Optional[float] = None,
+    t_max: Optional[float] = None,
     copy: bool = False,
 ):
 
     """\
     Subset the fitted tree.
 
+    if pseudotime parameter used, cutoff tree by removing cells/nodes after or before defined pseudotime.
+
     Parameters
     ----------
     adata
@@ -178,6 +182,18 @@ def subset_tree(
 
     adata = adata.copy() if copy else adata
 
+    if ((t_min is not None) | (t_max is not None)) & (mode != "pseudotime"):
+        logg.warn("setting mode to `pseudotime`")
+        mode = "pseudotime"
+    if mode == "pseudotime":
+        _subset_t(adata, t_min, t_max)
+        logg.info(
+            "    finished", time=True, end=" " if settings.verbosity > 2 else "\n"
+        )
+        logg.hint("tree subsetted")
+
+        return adata if copy else None
+
     adata.obs["old_milestones"] = adata.obs.milestones.copy()
 
     if "milestones_colors" in adata.uns:
@@ -202,7 +218,9 @@ def subset_tree(
     R = adata.obsm["X_R"].copy()
     F = graph["F"].copy()
 
-    dct = adata.uns["graph"]["milestones"]
+    dct = graph["milestones"]
+    dct_rev = dict(zip(dct.values(), dct.keys()))
+    oldmil = adata.obs.milestones.cat.categories.copy()
 
     edges = graph["pp_seg"][["from", "to"]].astype(str).apply(tuple, axis=1).values
     img = igraph.Graph()
@@ -211,10 +229,6 @@ def subset_tree(
     )
     img.add_edges(edges)
 
-    dct = graph["milestones"]
-    dct_rev = dict(zip(dct.values(), dct.keys()))
-    oldmil = adata.obs.milestones.cat.categories.copy()
-
     if "milestones_colors" in adata.uns:
         oldmilcol = np.array(adata.uns["milestones_colors"])
 
@@ -258,6 +272,9 @@ def subset_tree(
     tips = np.argwhere(np.array(g.degree()) == 1).flatten()
     forks = np.argwhere(np.array(g.degree()) > 2).flatten()
 
+    Rsum = R.sum(axis=1)
+    norm_R_cpu(R, Rsum)
+
     adata.uns["graph"]["tips"] = tips
     adata.uns["graph"]["forks"] = forks
     adata.uns["graph"]["B"] = B
@@ -273,6 +290,9 @@ def subset_tree(
 
     del adata.uns["graph"]["milestones"]
     del adata.obs["milestones"]
+    del adata.uns["graph"]["root"]
+    if "root2" in adata.uns["graph"]:
+        del adata.uns["graph"]["root2"]
     root(adata, nodes[dct[rmil]])
     pseudotime(adata)
 
@@ -313,6 +333,31 @@ def subset_tree(
     return adata if copy else None
 
 
+def _subset_t(adata: AnnData, t_min=None, t_max=None):
+    t_min = adata.obs.t.min() - 0.1 if t_min is None else t_min
+    t_max = adata.obs.t.max() + 0.1 if t_max is None else t_max
+    pp_info = adata.uns["graph"]["pp_info"]
+    keep = pp_info.index[(pp_info.time > t_min) & (pp_info.time < t_max)]
+    pp_info = pp_info.loc[keep]
+    adata.uns["graph"]["B"] = adata.uns["graph"]["B"][keep, :][:, keep]
+    adata.uns["graph"]["F"] = adata.uns["graph"]["F"][:, keep]
+    R = adata.obsm["X_R"][:, keep]
+    Rsum = R.sum(axis=1)
+    norm_R_cpu(R, Rsum)
+    adata.obsm["X_R"] = R
+    g = igraph.Graph.Adjacency(
+        (adata.uns["graph"]["B"] > 0).tolist(), mode="undirected"
+    )
+    adata.uns["graph"]["tips"] = np.argwhere(np.array(g.degree()) == 1).ravel()
+    adata.uns["graph"]["forks"] = np.argwhere(np.array(g.degree()) > 2).ravel()
+    adata.uns["graph"]["pp_info"] = pp_info.reset_index(drop=True)
+    root(adata, adata.uns["graph"]["pp_info"].time.idxmin())
+    adata._inplace_subset_obs(
+        adata.obs_names[(adata.obs.t > t_min) & (adata.obs.t < t_max)]
+    )
+    pseudotime(adata)
+
+
 def attach_tree(
     adata: AnnData,
     adata_branch: AnnData,
@@ -456,6 +501,9 @@ def extend_tip(adata, tip, use_rep, restrict_seg):
     selecta = adata.obsm["X_R"].argmax(axis=1) == tip
     adata_sub = adata[selecta]
 
+    if sum(selecta) == 0:
+        return adata
+
     early_cell = (
         adata_sub.obs.t.idxmin() if tip == graph["root"] else adata_sub.obs.t.idxmax()
     )
@@ -585,6 +633,98 @@ def extend_tips(adata: AnnData, restrict_seg: bool = False, copy: bool = False):
     return adata if copy else None
 
 
+def simplify(adata: AnnData, n_nodes: int = 10, copy: bool = False):
+
+    """\
+    While keeping nodes defining forks and tips (milestones), reduce the number of nodes composing the segments.
+
+    This can be helpful to simplify visualisations.
+
+    Parameters
+    ----------
+    adata
+        Annotated data matrix.
+    n_nodes
+        Number of nodes to keep per segments in between milestone nodes.
+    copy
+        Return a copy instead of writing to adata.
+
+    Returns
+    -------
+    adata : anndata.AnnData
+        Dataset with simplified graph.
+
+    """
+
+    logg.info("simplifying graph", reset=True)
+
+    adata = adata.copy() if copy else adata
+
+    use_rep = adata.uns["graph"]["use_rep"]
+    metric = adata.uns["graph"]["metrics"]
+    res = []
+    miles = {}
+    mils = np.array(list(adata.uns["graph"]["milestones"].values()))
+    R = adata.obsm["X_R"]
+    pp_info = adata.uns["graph"]["pp_info"]
+    for s in pp_info.seg.unique():
+        pp_info = adata.uns["graph"]["pp_info"]
+        pp_info = pp_info.loc[pp_info.seg == s]
+        pp_info.sort_values("time", inplace=True)
+
+        tokeep = pp_info.index[np.isin(pp_info.index, mils)]
+        tosimplify = pp_info.index[~np.isin(pp_info.index, mils)]
+        for tk in tokeep:
+            res.append(R[:, tk])
+            miles[tk] = len(res) - 1
+        idxs = np.array_split(tosimplify, n_nodes)
+        for idx in idxs:
+            res.append(R[:, idx].mean(axis=1))
+
+    newR = np.vstack(res).T
+    newR = newR / newR.sum(axis=1).reshape(-1, 1)
+
+    rep = adata.obsm[use_rep]
+    F_mat = np.dot(rep.T, newR) / newR.sum(axis=0)
+
+    d = pairwise_distances(F_mat.T, metric=metric)
+
+    csr = csr_matrix(np.triu(d, k=-1))
+    Tcsr = minimum_spanning_tree(csr)
+    mat = Tcsr.toarray()
+    mat = mat + mat.T - np.diag(np.diag(mat))
+    B = (mat > 0).astype(int)
+
+    g = igraph.Graph.Adjacency((B > 0).tolist(), mode="undirected")
+    tips = np.argwhere(np.array(g.degree()) == 1).flatten()
+    forks = np.argwhere(np.array(g.degree()) > 2).flatten()
+
+    adata.obsm["X_R"] = newR
+    adata.uns["graph"]["B"] = B
+    adata.uns["graph"]["F"] = F_mat
+    adata.uns["graph"]["tips"] = tips
+    adata.uns["graph"]["forks"] = forks
+
+    newmil = {}
+    for m in adata.uns["graph"]["milestones"].keys():
+        newmil[m] = miles[adata.uns["graph"]["milestones"][m]]
+
+    milroot = (
+        pd.Series(adata.uns["graph"]["milestones"]) == adata.uns["graph"]["root"]
+    ).idxmax()
+    newroot = newmil[milroot]
+
+    root(adata, newroot)
+    pseudotime(adata)
+
+    adata.uns["graph"]["milestones"] = newmil
+
+    logg.info("    finished", time=True, end=" " if settings.verbosity > 2 else "\n")
+    logg.hint("graph simplified")
+
+    return adata if copy else None
+
+
 def getpath(adata, root_milestone, milestones, include_root=False):
     """\
     Obtain dataframe of cell of a given path.
